@@ -14,8 +14,10 @@
 #    under the License.
 
 import tempfile
+from unittest import mock
 
 from cinder.db.sqlalchemy import api as sqla_api
+from cinder.db.sqlalchemy import models as sqla_models
 from cinder import objects as cinder_ovos
 from oslo_db import api as oslo_db_api
 
@@ -30,11 +32,10 @@ class TestDBPersistence(base.BasePersistenceTest):
                        'connection': CONNECTION}
 
     def tearDown(self):
-        sqla_api.model_query(self.context, sqla_api.models.Snapshot).delete()
+        sqla_api.model_query(self.context, sqla_models.Snapshot).delete()
         sqla_api.model_query(self.context,
-                             sqla_api.models.VolumeAttachment).delete()
-        sqla_api.model_query(self.context,
-                             sqla_api.models.Volume).delete()
+                             sqla_models.VolumeAttachment).delete()
+        sqla_api.model_query(self.context, sqla_models.Volume).delete()
         sqla_api.get_session().query(dbms.KeyValue).delete()
         super(TestDBPersistence, self).tearDown()
 
@@ -60,6 +61,8 @@ class TestDBPersistence(base.BasePersistenceTest):
 
     def test_set_snapshot(self):
         vol = cinderlib.Volume(self.backend, size=1, name='disk')
+        # This will assign a volume type, which is necessary for the snapshot
+        vol.save()
         snap = cinderlib.Snapshot(vol, name='disk')
 
         self.assertEqual(0, len(sqla_api.snapshot_get_all(self.context)))
@@ -118,6 +121,47 @@ class TestDBPersistence(base.BasePersistenceTest):
                               cinder_ovos.VolumeType)
         self.assertEqual('__DEFAULT__', self.persistence.DEFAULT_TYPE.name)
 
+    def test_delete_volume_with_metadata(self):
+        vols = self.create_volumes([{'size': i, 'name': 'disk%s' % i,
+                                     'metadata': {'k': 'v', 'k2': 'v2'},
+                                     'admin_metadata': {'k': '1'}}
+                                    for i in range(1, 3)])
+        self.persistence.delete_volume(vols[0])
+        res = self.persistence.get_volumes()
+        self.assertListEqualObj([vols[1]], res)
+
+        for model in (dbms.models.VolumeMetadata,
+                      dbms.models.VolumeAdminMetadata):
+            query = dbms.sqla_api.model_query(self.context, model)
+            res = query.filter_by(volume_id=vols[0].id).all()
+            self.assertEqual([], res)
+
+
+class TestDBPersistenceNewerSchema(base.helper.TestHelper):
+    """Test DBMS plugin can start when the DB has a newer schema."""
+    CONNECTION = 'sqlite:///' + tempfile.NamedTemporaryFile().name
+    PERSISTENCE_CFG = {'storage': 'db',
+                       'connection': CONNECTION}
+
+    @classmethod
+    def setUpClass(cls):
+        pass
+
+    def _raise_exc(self):
+        inner_exc = dbms.migrate.exceptions.VersionNotFoundError()
+        exc = dbms.exception.DBMigrationError(inner_exc)
+        self.original_db_sync()
+        raise(exc)
+
+    def test_newer_db_schema(self):
+        self.original_db_sync = dbms.migration.db_sync
+        with mock.patch.object(dbms.migration, 'db_sync',
+                               side_effect=self._raise_exc) as db_sync_mock:
+            super(TestDBPersistenceNewerSchema, self).setUpClass()
+            db_sync_mock.assert_called_once()
+            self.assertIsInstance(cinderlib.Backend.persistence,
+                                  dbms.DBPersistence)
+
 
 class TestMemoryDBPersistence(TestDBPersistence):
     PERSISTENCE_CFG = {'storage': 'memory_db'}
